@@ -31,10 +31,10 @@ import sipconfig
 
 
 # Initialise the globals.
-pyqt_version = 0x040001
-pyqt_version_str = "4.0.1"
+pyqt_version = 0x040100
+pyqt_version_str = "4.1"
 
-sip_min_version = 0x040404
+sip_min_version = 0x040500
 
 qt_version = 0
 qt_edition = ""
@@ -66,6 +66,7 @@ opt_concat = 0
 opt_split = 1
 opt_tracing = 0
 opt_verbose = 0
+opt_api = 0
 
 opt_vendorcheck = 0
 opt_vendincdir = sipcfg.py_inc_dir
@@ -83,9 +84,10 @@ def usage(rcode = 2):
         def_qmake = "none"
 
     print "Usage:"
-    print "    python configure.py [-h] [-b dir] [-c] [-d dir] [-i] [-j #] [-k] [-l dir] [-m dir] [-q exe] [-r] [-u] [-v dir] [-w] option=value option+=value ..."
+    print "    python configure.py [-h] [-a] [-b dir] [-c] [-d dir] [-i] [-j #] [-k] [-l dir] [-m dir] [-q exe] [-r] [-u] [-v dir] [-w] option=value option+=value ..."
     print "where:"
     print "  -h      display this help message"
+    print "  -a      install the PyQt API file for QScintilla [default only if QScintilla installed]"
     print "  -b dir  where pyuic4, pyrcc4 and pylupdate4 will be installed [default %s]" % opt_pyqtbindir
     print "  -c      concatenate each module's C++ source files"
     print "  -d dir  where the PyQt4 modules will be installed [default %s]" % opt_pyqtmoddir
@@ -115,19 +117,22 @@ class ConfigurePyQt4:
             0x040101: None,
             0x040102: "Qt_4_1_1",
             0x040103: "Qt_4_1_2",
-            0x050000: "Qt_4_1_3"
+            0x040200: "Qt_4_1_3",
+            0x050000: "Qt_4_2_0"
         }
 
     def check_modules(self):
         pyqt_modules.append("QtCore")
 
-        check_module("QtGui", "qwidget.h", "QWidget()")
-        check_module("QtNetwork", "qhostaddress.h", "QHostAddress()")
-        check_module("QtOpenGL", "qgl.h", "QGLWidget()")
-        check_module("QtSql", "qsqldatabase.h", "QSqlDatabase()")
-        check_module("QtSvg", "qsvgwidget.h", "QSvgWidget()")
-        check_module("QtXml", "qdom.h", "QDomDocument()")
-        check_module("QtAssistant", "qassistantclient.h", "QAssistantClient(\"foo\")")
+        check_module("QtGui", "qwidget.h", "new QWidget()")
+        check_module("QtNetwork", "qhostaddress.h", "new QHostAddress()")
+        check_module("QtOpenGL", "qgl.h", "new QGLWidget()")
+        check_module("QtSql", "qsqldatabase.h", "new QSqlDatabase()")
+        check_module("QtSvg", "qsvgwidget.h", "new QSvgWidget()")
+        check_module("QtTest", "QtTest", "QTest::qSleep(0)")
+        check_module("QtXml", "qdom.h", "new QDomDocument()")
+        check_module("QtAssistant", "qassistantclient.h", "new QAssistantClient(\"foo\")")
+        check_module("QAxContainer", "qaxobject.h", "new QAxObject()", "QAxContainer")
 
     def code(self):
         # Note that the order of the dependent modules is important.
@@ -152,12 +157,18 @@ class ConfigurePyQt4:
         if "QtSql" in pyqt_modules:
             generate_code("QtSql", ["QtSql", "QtGui", "QtCore"])
 
+        if "QtTest" in pyqt_modules:
+            generate_code("QtTest", ["QtTest", "QtGui", "QtCore"])
+
         if "QtXml" in pyqt_modules:
             generate_code("QtXml", ["QtXml", "QtCore"])
 
         if "QtAssistant" in pyqt_modules:
             generate_code("QtAssistant", ["QtAssistant", "QtNetwork", "QtGui", "QtCore"])
 
+        if "QAxContainer" in pyqt_modules:
+            generate_code("QAxContainer", ["QAxContainer", "QtGui", "QtCore"])
+
         # Generate the consolidated module.
         sipconfig.inform("Creating Qt consolidated module...")
         f = open("Qt.py", "w")
@@ -167,22 +178,39 @@ class ConfigurePyQt4:
 
         f.close()
 
+        # Generate the QScintilla API file.
+        sipconfig.inform("Creating QScintilla API file...")
+        f = open("PyQt4.api", "w")
+
+        for m in pyqt_modules:
+            api = open(m + ".api")
+
+            for l in api:
+                f.write("PyQt4." + l)
+
+            api.close()
+            os.remove(m + ".api")
+
+        f.close()
+
     def module_installs(self):
         return ["__init__.py", "Qt.py", "pyqtconfig.py"]
 
     def tools(self):
         tool = []
 
-        sipconfig.inform("Creating elementtree Makefile...")
+        # Only include ElementTree for older versions of Python.
+        if sipcfg.py_version < 0x020500:
+            sipconfig.inform("Creating elementtree Makefile...")
 
-        makefile = sipconfig.PythonModuleMakefile(
-            configuration=sipcfg,
-            dstdir=os.path.join(opt_pyqtmoddir, "elementtree"),
-            dir="elementtree"
-        )
+            makefile = sipconfig.PythonModuleMakefile(
+                configuration=sipcfg,
+                dstdir=os.path.join(opt_pyqtmoddir, "elementtree"),
+                dir="elementtree"
+            )
 
-        makefile.generate()
-        tool.append("elementtree")
+            makefile.generate()
+            tool.append("elementtree")
 
         # Create the pyuic4 wrapper.  Use the GUI version on MacOS (so that
         # previews work properly and normal console use will work anyway), but
@@ -328,7 +356,7 @@ def run_command(cmd):
 
     try:
         os.wait()
-    except AttributeError:
+    except:
         pass
 
 
@@ -343,11 +371,12 @@ def remove_file(fname):
         pass
 
 
-def compile_qt_program(name, mname):
+def compile_qt_program(name, mname, lib=None):
     """Compile a simple Qt application.
 
     name is the name of the single source file.
     mname is the name of the Qt module.
+    lib is an optional extra library.
 
     Returns the name of the executable suitable for running or None if it
     wasn't created.
@@ -356,6 +385,9 @@ def compile_qt_program(name, mname):
 
     makefile = sipconfig.ProgramMakefile(sipcfg, console=1, qt=[mname], warnings=0, opengl=opengl, debug=opt_debug)
 
+    if lib:
+        makefile.extra_libs.append(lib)
+
     exe, build = makefile.build_command(name)
 
     # Make sure the executable file doesn't exist.
@@ -388,28 +420,30 @@ def check_vendorid():
             sipconfig.inform("vendorid.h could not be found in %s and so signed interpreter checking will be disabled. If the VendorID package is installed then use the -l argument to explicitly specify the correct directory." % opt_vendincdir)
 
 
-def check_module(mname, incfile, ctor):
+def check_module(mname, incfile, test, lib=None):
     """See if a module can be built and, if so, add it to the global list of
     modules.
 
     mname is the name of the module.
     incfile is the name of the include file needed for the test.
-    ctor is the C++ constructor of the class being used for the test.
+    test is a C++ statement being used for the test.
+    lib is an optional extra library.
     """
     # Check the module's main .sip file exists.
     if os.access(os.path.join("sip", mname, mname + "mod.sip"), os.F_OK):
         sipconfig.inform("Checking to see if the %s module should be built..." % mname)
 
-        if check_class(incfile, ctor, mname):
+        if check_api(incfile, test, mname, lib):
             pyqt_modules.append(mname)
 
 
-def check_class(incfile, ctor, mname):
+def check_api(incfile, test, mname, lib=None):
     """Return non-zero if a class is available.
 
     incfile is the name of the include file needed for the test.
-    ctor is the C++ constructor of the class.
+    test is a C++ statement being used for the test.
     mname is the name of the module.
+    lib is an optional extra library.
     """
     # We use a module specific name to avoid a potential problem on Windows
     # where the operating system doesn't delete previous tests quickly enough.
@@ -421,13 +455,13 @@ def check_class(incfile, ctor, mname):
 
 int main(int, char **)
 {
-    new %s;
+    %s;
 }
-""" % (incfile, ctor))
+""" % (incfile, test))
 
     f.close()
 
-    return compile_qt_program(cfgtest, mname)
+    return compile_qt_program(cfgtest, mname, lib)
 
 
 def set_sip_flags(pyqt):
@@ -467,6 +501,11 @@ def set_sip_flags(pyqt):
         qt_sip_flags.append("-t")
         qt_sip_flags.append(verstag)
 
+    # Handle the version specific Python features.
+    if sipcfg.py_version < 0x020400:
+        qt_sip_flags.append("-x")
+        qt_sip_flags.append("Py_DateTime")
+
 
 def generate_code(mname, qt_mods, extra_include_dir=None, extra_lib_dir=None, extra_lib=None):
     """Generate the code for a module.
@@ -503,6 +542,9 @@ def generate_code(mname, qt_mods, extra_include_dir=None, extra_lib_dir=None, ex
     if opt_tracing:
         argv.append("-r")
 
+    argv.append("-a")
+    argv.append(mname + ".api")
+
     argv.append("-c")
     argv.append(mname)
 
@@ -641,6 +683,9 @@ def get_build_macros(overrides):
 
         if not os.path.dirname(fname):
             fname = os.path.join(qt_datadir, "mkspecs", fname)
+    elif sys.platform == "darwin":
+        # The Qt Mac binary installer defaults to xcode which we don't want.
+        fname = os.path.join(qt_datadir, "mkspecs", "macx-g++")
     else:
         fname = os.path.join(qt_datadir, "mkspecs", "default")
 
@@ -833,7 +878,10 @@ def resolve_qt4_library(generator):
     qt_lib = "QtCore"
 
     if opt_debug and sys.platform != "win32":
-        qt_lib = qt_lib + "_debug"
+        # Qt v4.2 and later for X11 platforms doesn't have separate debug
+        # libraries.
+        if qt_version < 0x040200 or sys.platform == "darwin":
+            qt_lib = qt_lib + "_debug"
 
     if not is_qt_library(generator, qt_lib):
         sipconfig.error("The %s Qt library could not be found in %s." % (qt_lib, qt_libdir))
@@ -879,11 +927,14 @@ def get_qt_configuration():
     make_target = ""
     cpp_file = app + ".cpp"
     out_file = app + ".out"
+    qmake_args = "-o " + make_file
 
     if sys.platform == "win32":
         exe_file = os.path.join("release", app + ".exe")
         make_target = " release"
     elif sys.platform == "darwin":
+        # The Qt binary installer has macx-xcode as the default.
+        qmake_args = "-spec macx-g++ " + qmake_args
         exe_file = os.path.join(app + ".app", "Contents", "MacOS", app)
     else:
         exe_file = os.path.join(".", app)
@@ -954,7 +1005,7 @@ int main(int, char **)
 
     # Create the makefile, first making sure it doesn't already exist.
     remove_file(make_file)
-    run_command("%s -o %s %s" % (opt_qmake, make_file, pro_file))
+    run_command("%s %s %s" % (opt_qmake, qmake_args, pro_file))
 
     if not os.access(make_file, os.F_OK):
         sipconfig.error("%s failed to create a makefile. Make sure you have a working Qt v4 qmake on your PATH or use the -q argument to explicitly specify a working Qt v4 qmake." % opt_qmake)
@@ -1032,17 +1083,19 @@ def main(argv):
 
     # Parse the command line.
     try:
-        optlist, args = getopt.getopt(argv[1:], "hb:cd:g:ij:kl:m:q:ruv:w")
+        optlist, args = getopt.getopt(argv[1:], "ahb:cd:g:ij:kl:m:q:ruv:w")
     except getopt.GetoptError:
         usage()
 
     global opt_pyqtbindir, opt_pyqtmoddir, opt_pyqtsipdir, opt_static
     global opt_debug, opt_concat, opt_split, opt_tracing, opt_verbose
-    global opt_vendorcheck, opt_vendincdir, opt_vendlibdir
+    global opt_vendorcheck, opt_vendincdir, opt_vendlibdir, opt_api
 
     for opt, arg in optlist:
         if opt == "-h":
             usage(0)
+        elif opt == "-a":
+            opt_api = 1
         elif opt == "-b":
             opt_pyqtbindir = os.path.abspath(arg)
         elif opt == "-c":
@@ -1113,10 +1166,17 @@ def main(argv):
     # Create the additional Makefiles.
     sipconfig.inform("Creating top level Makefile...")
 
+    installs=[(pyqt.module_installs(), opt_pyqtmoddir)]
+
+    scidir = os.path.join(qt_datadir, "qsci")
+
+    if opt_api or os.path.isdir(scidir):
+        installs.append(("PyQt4.api", os.path.join(scidir, "api", "python")))
+
     sipconfig.ParentMakefile(
         configuration=sipcfg,
         subdirs=pyqt_modules + pyqt.tools(),
-        installs=(pyqt.module_installs(), opt_pyqtmoddir)
+        installs=installs
     ).generate()
 
     # Install the configuration module.
