@@ -7,7 +7,7 @@ QtCore = None
 QtGui = None
 
 def int_list(prop):
-    return [int(child.text) for child in iter(prop)]
+    return [int(child.text) for child in prop]
 
 bool_ = lambda v: v == "true"
 
@@ -16,12 +16,12 @@ def needsWidget(func):
     return func
 
 class Properties(object):
-    def __init__(self, qobject_creator, QtCore_mod, QtGui_mod):
+    def __init__(self, factory, QtCore_mod, QtGui_mod):
         self.reset()
         global QtGui, QtCore
         QtGui = QtGui_mod
         QtCore = QtCore_mod
-        self.qobject_creator = qobject_creator
+        self.factory = factory
 
     def reset(self):
         self.buddies = []
@@ -55,12 +55,15 @@ class Properties(object):
         return prop.text == 'true'
 
     def _string(self, prop):
-        if isinstance(prop.text, unicode):
-            text = prop.text.encode("UTF-8")
+        if prop.attrib.get("notr", None) == "true":
+            return self._cstring(prop)
         else:
-            text = prop.text
-        return QtGui.QApplication.translate(self.uiname, text,
-                                            None, QtGui.QApplication.UnicodeUTF8)
+            if prop.text is None:
+                return ""
+            text = prop.text.encode("UTF-8")
+            return QtGui.QApplication.translate(self.uiname, text,
+                                                None, QtGui.QApplication.UnicodeUTF8)
+
 
     def _cstring(self, prop):
         return str(prop.text)
@@ -75,10 +78,10 @@ class Properties(object):
         return QtCore.QSize(*int_list(prop))
 
     def _pixmap(self, prop):
-        return QtGui.QPixmap(prop.text)
+        return QtGui.QPixmap(prop.text.replace("\\", "\\\\"))
 
     def _iconset(self, prop):
-        return QtGui.QIcon(prop.text)
+        return QtGui.QIcon(prop.text.replace("\\", "\\\\"))
 
     def _cursor(self, prop):
         return QtGui.QCursor(QtCore.Qt.CursorShape(int(prop.text)))
@@ -93,6 +96,29 @@ class Properties(object):
     def _time(self, prop):
         return QtCore.QTime(*int_list(prop))
 
+    def _palette(self, prop):
+        palette = self.factory.createQObject("QPalette", "palette", (),
+                                                   is_attribute = False)
+        for palette_elem in prop:
+            sub_palette = getattr(QtGui.QPalette, palette_elem.tag.title())
+            for idx, color in enumerate(palette_elem):
+                palette.setColor(sub_palette, QtGui.QPalette.ColorRole(idx),
+                                 self._color(color))
+        return palette
+
+    #@needsWidget
+    def _sizepolicy(self, prop, widget):
+        values = [int(child.text) for child in prop]
+        sizePolicy = self.factory.createQObject(
+            "QSizePolicy", "sizePolicy",
+            (QtGui.QSizePolicy.Policy(values[0]), QtGui.QSizePolicy.Policy(values[1])),
+            is_attribute = False)
+        sizePolicy.setHorizontalStretch(values[2])
+        sizePolicy.setVerticalStretch(values[3])
+        sizePolicy.setHeightForWidth(widget.sizePolicy().hasHeightForWidth())
+        return sizePolicy
+    _sizepolicy = needsWidget(_sizepolicy)
+
     # font needs special handling/conversion of all child elements
     _font_attributes = (("Family",    str),
                         ("PointSize", int),
@@ -104,7 +130,7 @@ class Properties(object):
 
     #@needsWidget
     def _font(self, prop, widget):
-        newfont = self.qobject_creator.createQObject("QFont", "font",
+        newfont = self.factory.createQObject("QFont", "font",
                                                      (widget.font(), ),
                                                      is_attribute = False)
         for attr, converter in self._font_attributes:
@@ -155,6 +181,7 @@ class Properties(object):
             if prop[0].text is None:
                 continue
             propname = prop.attrib["name"]
+            logging.debug("setting property %s" % (propname,))
             if hasattr(self, propname):
                 getattr(self, propname)(widget, prop)
             else:
@@ -173,7 +200,11 @@ class Properties(object):
     def _setViaSetProperty(self, widget, prop):
         widget.setProperty(prop.attrib["name"], QtCore.QVariant(self.convert(prop)))
 
-    # all properties that need special handling
+
+    # SPECIAL PROPERTIES
+    # If a property has a well-known value type but needs special, context-dependent
+    # handling, the default behaviour can be overridden here
+
     currentIndex = currentRow = _delay
     showDropIndicator = intValue = value = _setViaSetProperty # QLCDValue needs this
 
@@ -182,6 +213,11 @@ class Properties(object):
     def buddy(self, widget, prop):
         self.buddies.append((widget, prop[0].text))
 
+    # not interested
+    # TODO: check if that works always!
+    def objectName(self, widget, prop):
+        pass
+    
     # geometry has to be handled specially if set on the toplevel widget
     def geometry(self, widget, prop):
         rect = self._rect(prop[0])
@@ -203,30 +239,6 @@ class Properties(object):
         else:
             widget.setOrientation(self._enum(prop[0]))
 
-    # sizePolicy is just plain weird
-    def sizePolicy(self, widget, prop):
-        values = [int(child.text) for child in prop[0]]
-        sizePolicy = self.qobject_creator.createQObject(
-            "QSizePolicy", "sizePolicy",
-            (QtGui.QSizePolicy.Policy(values[0]), QtGui.QSizePolicy.Policy(values[1])),
-            is_attribute = False)
-        sizePolicy.setHorizontalStretch(values[2])
-        sizePolicy.setVerticalStretch(values[3])
-        sizePolicy.setHeightForWidth(widget.sizePolicy().hasHeightForWidth())
-        widget.setSizePolicy(sizePolicy)
-
-    # palette needs a lot of function calls
-    def palette(self, widget, prop):
-        palette = self.qobject_creator.createQObject("QPalette", "palette", (),
-                                                   is_attribute = False)
-        for palette_elem in prop[0]:
-            sub_palette = getattr(QtGui.QPalette, palette_elem.tag.title())
-            for idx, color in enumerate(palette_elem):
-                palette.setColor(sub_palette, QtGui.QPalette.ColorRole(idx),
-                                 self._color(color))
-        widget.setPalette(palette)
-
-
     # attribute isWrapping of QListView is named inconsistently,
     # should be wrapping
     def isWrapping(self, widget, prop):
