@@ -4,15 +4,20 @@ import re
 from itertools import count
 
 try:
-    from cElementTree import parse
+    from xml.etree.cElementTree import parse
 except ImportError:
     try:
-        from elementtree.ElementTree import parse
+        from cElementTree import parse
     except ImportError:
-        from PyQt4.elementtree.ElementTree import parse
+        try:
+            from elementtree.ElementTree import parse
+        except ImportError:
+            from PyQt4.elementtree.ElementTree import parse
         
 
 from exceptions import NoSuchWidgetError
+from properties import Properties
+from objcreator import QObjectCreator
 
 if sys.version_info < (2,4,0):
     def reversed(seq):
@@ -80,11 +85,15 @@ class WidgetStack(list):
 
 
 class UIParser(object):    
-    def __init__(self, QtCoreModule, QtGuiModule):
-        self.reset()
+    def __init__(self, QtCoreModule, QtGuiModule, creatorPolicy):
+        self.factory = QObjectCreator(creatorPolicy)
+        self.wprops = Properties(self.factory, QtCoreModule, QtGuiModule)
+        
         global QtCore, QtGui
         QtCore = QtCoreModule
         QtGui = QtGuiModule
+        
+        self.reset()
 
     def uniqueName(self, name):
         """UIParser.uniqueName(string) -> string
@@ -111,15 +120,16 @@ class UIParser(object):
         self.name_suffixes = {}
         self.defaults = {"spacing": 6, "margin": 0}
         self.actions = []
-        
+        self.currentActionGroup = None
+        self.resources = []
 
     def setupObject(self, clsname, parent, branch, is_attribute = True):
-        name = self.uniqueName(branch.attrib["name"] or clsname[1:].lower())
+        name = self.uniqueName(branch.attrib.get("name") or clsname[1:].lower())
         if parent is None:
             args = ()
         else:
             args = (parent, )
-        obj =  self.wdb.createQObject(clsname, name, args, is_attribute)
+        obj =  self.factory.createQObject(clsname, name, args, is_attribute)
         self.wprops.setProperties(obj, branch)
         obj.setObjectName(name)
         if is_attribute:
@@ -160,13 +170,33 @@ class UIParser(object):
 
             
         if isinstance(self.stack.topwidget, QtGui.QToolBox):
-            self.stack.topwidget.addItem(widget, self.wprops.getAttribute(elem, "label"))
+            icon = self.wprops.getAttribute(elem, "icon")
+            if icon is not None:
+                self.stack.topwidget.addItem(widget, icon, self.wprops.getAttribute(elem, "label"))
+            else:
+                self.stack.topwidget.addItem(widget, self.wprops.getAttribute(elem, "label"))
+                
+            tooltip = self.wprops.getAttribute(elem, "toolTip")
+            if tooltip is not None:
+                self.stack.topwidget.setItemToolTip(self.stack.topwidget.indexOf(widget), tooltip)
+                
         elif isinstance(self.stack.topwidget, QtGui.QTabWidget):
-            self.stack.topwidget.addTab(widget, self.wprops.getAttribute(elem, "title"))
+            icon = self.wprops.getAttribute(elem, "icon")
+            if icon is not None:
+                self.stack.topwidget.addTab(widget, icon, self.wprops.getAttribute(elem, "title"))
+            else:
+                self.stack.topwidget.addTab(widget, self.wprops.getAttribute(elem, "title"))
+
+            tooltip = self.wprops.getAttribute(elem, "toolTip")
+            if tooltip is not None:
+                self.stack.topwidget.setTabToolTip(self.stack.topwidget.indexOf(widget), tooltip)
+            
         elif isinstance(self.stack.topwidget, QtGui.QStackedWidget):
             self.stack.topwidget.addWidget(widget)
+            
         elif isinstance(self.stack.topwidget, QtGui.QDockWidget):
             self.stack.topwidget.setWidget(widget)
+            
         elif isinstance(self.stack.topwidget, QtGui.QMainWindow):
             if type(widget) == QtGui.QWidget:
                 self.stack.topwidget.setCentralWidget(widget)
@@ -192,7 +222,7 @@ class UIParser(object):
                   getattr(QtGui.QSizePolicy, sizeType.split("::")[-1]))
         if self.wprops.getProperty(elem, "orientation") == "Qt::Horizontal":
             policy = policy[1], policy[0]
-        spacer = self.wdb.createQObject("QSpacerItem", self.uniqueName("spacerItem"),
+        spacer = self.factory.createQObject("QSpacerItem", self.uniqueName("spacerItem"),
                                        (width, height) + policy,
                                         is_attribute = False)
         if self.stack.topIsLayout():
@@ -229,7 +259,7 @@ class UIParser(object):
                 if self.first_item:
                     w.clear()
                     self.first_item = False
-                item = self.wdb.createQObject("QListWidgetItem",
+                item = self.factory.createQObject("QListWidgetItem",
                                               self.uniqueName("item"),
                                               (w,), False)
                 self.wprops.setProperties(item, elem)
@@ -238,7 +268,7 @@ class UIParser(object):
                     w.clear()
                     self.first_item = False
                     self.itemstack = [w]
-                item = self.wdb.createQObject("QTreeWidgetItem",
+                item = self.factory.createQObject("QTreeWidgetItem",
                                               self.uniqueName("item"),
                                               (self.itemstack[-1],), False)
                 column = -1
@@ -252,7 +282,7 @@ class UIParser(object):
                 self.traverseWidgetTree(elem)
                 self.itemstack.pop()
             elif isinstance(w, QtGui.QTableWidget):
-                item = self.wdb.createQObject("QTableWidgetItem",
+                item = self.factory.createQObject("QTableWidgetItem",
                                               self.uniqueName("item"),
                                               (), False)
                 self.wprops.setProperties(item, elem)
@@ -276,7 +306,7 @@ class UIParser(object):
         elif isinstance(self.stack.topwidget, QtGui.QTableWidget):
             if len(elem) == 0:
                 return
-            item = self.wdb.createQObject("QTableWidgetItem",
+            item = self.factory.createQObject("QTableWidgetItem",
                                           self.uniqueName("headerItem"),
                                           (), False)
             self.wprops.setProperties(item, elem)
@@ -289,9 +319,16 @@ class UIParser(object):
             
 
     def createAction(self, elem):
-        self.setupObject("QAction", self.toplevelWidget, elem)
+        self.setupObject("QAction", self.currentActionGroup or self.toplevelWidget,
+                         elem)
 
+    def createActionGroup(self, elem):
+        action_group = self.setupObject("QActionGroup", self.toplevelWidget, elem)
+        self.currentActionGroup = action_group
+        self.traverseWidgetTree(elem)
+        self.currentActionGroup = None
         
+
     widgetTreeItemHandlers = {
         "widget"    : createWidget,
         "addaction" : addAction,
@@ -299,6 +336,7 @@ class UIParser(object):
         "spacer"    : createSpacer,
         "item"      : handleItem,
         "action"    : createAction,
+        "actiongroup": createActionGroup,
         "column"    : addHeader,
         "row"       : addHeader,
         }
@@ -312,8 +350,21 @@ class UIParser(object):
             
 
     def createUserInterface(self, elem):
-        self.toplevelWidget = self.createToplevelWidget(elem.attrib["class"])
-        self.toplevelWidget.setObjectName(self.uiname)
+        # Get the names of the class and widget.
+        cname = elem.attrib["class"]
+        wname = elem.attrib["name"]
+
+        # If there was no widget name then derive it from the class name.
+        if not wname:
+            wname = cname
+
+            if wname.startswith("Q"):
+                wname = wname[1:]
+
+            wname = wname[0].lower() + wname[1:]
+
+        self.toplevelWidget = self.createToplevelWidget(cname, wname)
+        self.toplevelWidget.setObjectName(wname)
         DEBUG("toplevel widget is %s",
               self.toplevelWidget.metaObject().className())
         self.wprops.setProperties(self.toplevelWidget, elem)
@@ -333,7 +384,7 @@ class UIParser(object):
                 action_obj = getattr(self.toplevelWidget, action_name)
                 if isinstance(action_obj, QtGui.QMenu):
                     widget.addAction(action_obj.menuAction())
-                else:
+                elif not isinstance(action_obj, QtGui.QActionGroup):
                     widget.addAction(action_obj)
 
 
@@ -354,9 +405,14 @@ class UIParser(object):
 
     def classname(self, elem):
         DEBUG("uiname is %s", elem.text)
-        self.uiname = elem.text
-        self.wprops.uiname = elem.text
-        self.setContext(self.uiname)
+        name = elem.text
+
+        if name is None:
+            name = ""
+
+        self.uiname = name
+        self.wprops.uiname = name
+        self.setContext(name)
 
 
     def setContext(self, context):
@@ -370,7 +426,7 @@ class UIParser(object):
     def readDefaults(self, elem):
         self.defaults["margin"] = int(elem.attrib["margin"])
         self.defaults["spacing"] = int(elem.attrib["spacing"])
-        
+
 
     def setTaborder(self, elem):
         try:
@@ -383,6 +439,21 @@ class UIParser(object):
             lastwidget = widget
 
 
+    def readResources(self, elem):
+        """
+        Read a "resources" tag and add the module to import to the parser's
+        list of them.
+        """
+        for include in elem.getiterator("include"):
+            loc = include.attrib.get("location")
+
+            # Assume our convention for naming the Python files generated by
+            # pyrcc4.
+            if loc and loc.endswith('.qrc'):
+                loc = loc[:-4] + '_rc'
+                self.resources.append(loc)
+
+
     def createConnections(self, elem):
         def name2object(obj):
             if obj == self.uiname:
@@ -392,7 +463,7 @@ class UIParser(object):
         for conn in iter(elem):
             QtCore.QObject.connect(name2object(conn.findtext("sender")),
                                    QtCore.SIGNAL(conn.findtext("signal")),
-                                   self.wdb.getSlot(name2object(conn.findtext("receiver")),
+                                   self.factory.getSlot(name2object(conn.findtext("receiver")),
                                                     conn.findtext("slot").split("(")[0]))
         QtCore.QMetaObject.connectSlotsByName(self.toplevelWidget)
 
@@ -414,17 +485,17 @@ class UIParser(object):
             classname = custom_widget.findtext("class")
             if classname.startswith("Q3"):
                 raise NoSuchWidgetError, classname
-            self.wdb.addCustomWidget(classname,
+            self.factory.addCustomWidget(classname,
                                      custom_widget.findtext("extends") or "QWidget",
                                      header2module(custom_widget.findtext("header")))
                                      
 
-    def createToplevelWidget(*args):
-        raise NotImplementedError, "must be overwritten"
+    def createToplevelWidget(self, classname, widgetname):
+        raise NotImplementedError, "must be overridden"
 
     
-    # finalize will be called after the whole tree has been parsed
-    # and can be overwritten
+    # finalize will be called after the whole tree has been parsed and can be
+    # overridden.
     def finalize(self):
         pass
 
@@ -439,6 +510,7 @@ class UIParser(object):
             ("widget",        self.createUserInterface),
             ("connections",   self.createConnections),
             ("tabstops",      self.setTaborder),
+            ("resources",     self.readResources),
         )
         document = parse(filename)
         version = document.getroot().attrib["version"]
